
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; MODULE      : tmxml.scm
;; DESCRIPTION : conversion of TeXmacs trees into Xml trees
;; COPYRIGHT   : (C) 2002  Joris van der Hoeven
;;
;; This software falls under the GNU general public license and comes WITHOUT
;; ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for details.
;; If you don't have this file, write to the Free Software Foundation, Inc.,
;; 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(texmacs-module (convert tmxml tmxml)
  (:use (convert tmxml xmlout))
  (:export tmxml-produce tmxml))

(define (tmxml-file x)
  (define (tmxml-keep? x)
    (not (func? x 'TeXmacs)))
  `(*TOP*
    (*PI* xml "version=\"1.0\"")
    (TeXmacs
     (@ (version ,(texmacs-version)) (xmlns:tm "http://www.texmacs.org"))
     (!stacked ,@(map tmxml (list-filter (cdr x) tmxml-keep?))))))

(define (tmxml-document l)
  (cons '!document (map (lambda (x) (list 'tm-par (tmxml x))) l)))

(define (tmxml-concat l)
  (cons '!concat (map tmxml l)))

(define (tmxml-with l)
  (tmxml-regular 'with (list (cons 'attr (cDr l)) (cAr l))))

(define (tmxml-args l)
  (cond ((null? l) l)
	((and (null? (cdr l)) (not (== (car l) ""))) (list (tmxml (car l))))
	(else (map (lambda (x) (list 'tm-arg (tmxml x))) l))))

(define (tmxml-apply dyn tag l)
  (cond ((and (pair? l) (func? (car l) 'attr))
	 (tmxml (cons* tag (cons* 'attr "tm-dyn" dyn (cdar l)) (cdr l))))
	(else (tmxml (cons* tag (list 'attr "tm-dyn" dyn) l)))))

(define (tmxml-group-attrs attrs)
  (if (< (length attrs) 2) '()
      (cons (list (string->symbol (tm->xml-name (car attrs))) (cadr attrs))
	    (tmxml-group-attrs (cddr attrs)))))

(define (tmxml-attrs l)
  (if (list-and (map string? l))
      (cons '@ (tmxml-group-attrs l))
      (cons 'tm-attr (map (lambda (x) (list 'tm-arg (tmxml x))) l))))

(define (tmxml-regular tag* l)
  (with tag (string->symbol (tm->xml-name (symbol->string tag*)))
    (cond ((and (pair? l) (func? (car l) 'attr))
	   (cons* tag (tmxml-attrs (cdar l)) (tmxml-args (cdr l))))
	  (else (cons tag (tmxml-args l))))))

(define (tmxml x)
  (cond ((string? x) (tm->xml-cdata x))
	((func? x '!file) (tmxml-file (cadr x)))
	((func? x 'document) (tmxml-document (cdr x)))
	((func? x 'concat) (tmxml-concat (cdr x)))
	((func? x 'with) (tmxml-with (cdr x)))
	((and (or (func? x 'apply) (func? x 'expand)) (string? (cadr x)))
	 (tmxml-apply (symbol->string (car x))
		      (string->symbol (cadr x))
		      (cddr x)))
	(else (tmxml-regular (car x) (cdr x)))))

(define (tmxml-simplify x)
  (cond ((not (pair? x)) x)
	((and (func? x 'quote) (string? (cadr x))) (cadr x))
	(else (map tmxml-simplify x))))

(define (tmxml-consistency-check-1 x)
  (with y (xmltm (tmxml x))
    (if (== x y)
	(display "ok\n")
	(write-diff x y))))

(define (tmxml-consistency-check nr orig new)
  (display* "Consistency check " nr ": ")
  (if (== orig new)
      (display "ok\n")
      (begin
	(display "failed\n")
	(write-diff orig new))))

(define (tmxml-produce x)
  ;; Convert a TeXmacs tree x to a XML tree.
  ;(display* (tmxml x) "\n")
  (let* ((simplified (tree->object (tree-simplify (object->tree x))))
	 (xml-tree (tmxml simplified))
	 (xml-doc (xmlout-produce xml-tree)))
    ;(tmxml-consistency-check 1 simplified (xmltm xml-tree))
    ;(tmxml-consistency-check 2 simplified (xml->object xml-doc))
    xml-doc))
