
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; MODULE      : init-texmacs.scm
;; DESCRIPTION : This is the standard TeXmacs initialization file
;; COPYRIGHT   : (C) 1999  Joris van der Hoeven
;;
;; This software falls under the GNU general public license and comes WITHOUT
;; ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for details.
;; If you don't have this file, write to the Free Software Foundation, Inc.,
;; 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;(display "Booting TeXmacs core functionality\n")
(load (url-concretize "$TEXMACS_PATH/progs/core/boot/boot.scm"))
(inherit-modules (core boot compat) (core boot abbrevs) (core boot debug)
		 (core boot srfi) (core boot ahash-table) (core boot prologue))
(inherit-modules (core regexp regexp-match))
(inherit-modules (core drd drd-rules) (core drd drd-query) (core drd drd-data))
(inherit-modules (core texmacs tm-define) (core texmacs tm-preferences)
		 (core texmacs tm-plugins) (core texmacs tm-modes)
		 (core texmacs tm-secure))
(inherit-modules (core library base) (core library list))
(inherit-modules (core gui menu-define) (core gui menu-widget)
		 (core gui kbd-define))
(inherit-modules (core tools tm-edit) (core tools tm-misc))
(retrieve-preferences)

;(display "Booting TeXmacs primitives\n")
(use-modules (texmacs texmacs tm-server) (texmacs texmacs tm-files)
	     (texmacs texmacs tm-print) (texmacs texmacs tm-view)
	     (texmacs texmacs tm-document) (texmacs texmacs tm-help))
(use-modules (texmacs tools tm-cursor) (texmacs tools tm-select)
	     (texmacs tools tm-circulate))
(use-modules (texmacs edit edit-preamble) (texmacs edit edit-format)
	     (texmacs edit edit-text) (texmacs edit edit-table)
	     (texmacs edit edit-fold) (texmacs edit edit-misc)
	     (texmacs edit edit-hybrid))
(re-export safely-kill-window)

;(display "Booting menus\n")
(lazy-menu (menus menu-file)
	   file-menu go-menu
	   new-file-menu load-menu save-menu print-menu close-menu)
(lazy-menu (menus menu-edit) edit-menu)
(lazy-menu (menus menu-insert)
	   insert-menu insert-table-menu insert-link-menu insert-switch-menu
	   insert-image-menu insert-page-insertion-menu position-float-menu)
(lazy-menu (menus menu-text)
	   text-menu texmacs-text-icons text-modifier-icons)
(lazy-menu (menus menu-mathematics)
	   mathematics-menu texmacs-math-icons math-modifier-icons)
(lazy-menu (menus menu-prog)
	   prog-modifier-icons session-menu
	   texmacs-session-icons texmacs-session-help-icons texmacs-help-icons)
(lazy-menu (menus menu-table) table-menu texmacs-table-icons)
(lazy-menu (menus menu-format) color-menu paragraph-menu format-menu)
(lazy-menu (menus menu-document)
	   document-menu project-menu document-style-menu global-language-menu)
(lazy-menu (menus menu-view) view-menu)
(lazy-menu (menus menu-tools) tools-menu)
(lazy-menu (menus menu-help) help-menu)
(use-modules (menus menu-preferences) (menus menu-main))

;(display "Booting keyboard\n")
;Note: lazyness implies preferences initialization to take place first
(use-modules (keyboard kbd-config))
(lazy-in-mode (keyboard kbd-wildcards) always?)
(if (like-emacs?) (lazy-in-mode (keyboard kbd-emacs) always?))
(if (like-old?) (lazy-in-mode (keyboard kbd-old) always?))
(lazy-in-mode (keyboard kbd-latex) always?)
(lazy-in-mode (keyboard kbd-general) always?)
(lazy-in-mode (keyboard kbd-preamble) always?)
(lazy-in-mode (keyboard kbd-text) in-text?)
(lazy-in-mode (keyboard kbd-math) in-math?)
(lazy-in-mode (keyboard kbd-table) in-table?)

;(display "Booting fonts\n")
(if (use-ec-fonts?)
    (use-modules (fonts fonts-ec))
    (use-modules (fonts fonts-cm)))
(use-modules (fonts fonts-adobe) (fonts fonts-x) (fonts fonts-math)
	     (fonts fonts-foreign) (fonts fonts-misc))

;(display "Booting converters\n")
(use-modules (convert tmtm tmtm-eqns))
(lazy-define (convert tmtex tmtex) tmtex-produce)
(lazy-define (convert tmxml tmxml) tmxml-produce)
(lazy-define (convert xmltm xmltm) xmltm)
(lazy-define (convert xmltm xmltm) xml->object)
(lazy-define (convert xmltm xmltm) xml->document)
(lazy-define (convert tmhtml tmhtml-expand) tmhtml-env-patch)
(lazy-define (convert tmhtml tmhtml) tmhtml-produce)
(lazy-define (convert htmltm htmltm) html->tree)
(lazy-define (convert htmltm htmltm) html->document)
(lazy-define (convert doc tmdoc) tmdoc-expand-help)
(lazy-define (convert doc docgrep) docgrep-in-doc)
(lazy-define (convert doc docgrep) docgrep-in-src)
(lazy-define (convert doc docgrep) docgrep-in-texts)
(lazy-define (convert doc tmdoc) tmdoc-expand-this)
(lazy-define (drd latex latex-drd) latex-arity)
(lazy-define (drd latex latex-drd) latex-type)

;(display "Booting plugins\n")
(lazy-define (texmacs plugin plugin-cmd) verbatim-serialize)
(lazy-define (texmacs plugin plugin-cmd) plugin-serialize)
(lazy-define (texmacs plugin plugin-cmd) format-command)
(if (url-exists? "$TEXMACS_HOME_PATH/system/setup.scm")
    (set! plugin-old-data-table
	  (load-object "$TEXMACS_HOME_PATH/system/setup.scm")))
(for-each plugin-initialize (plugin-list))
(if (not (== plugin-old-data-table plugin-data-table))
    (save-object "$TEXMACS_HOME_PATH/system/setup.scm" plugin-data-table))

;(display "Initializing user preferences\n")
(apply-preferences)
(menu-extend page-setup-menu ,@(compute-preferences-menu page-setup-tree))
(menu-extend preferences-menu ,@(compute-preferences-menu preferences-tree))
;(display "Initialization done\n")
