
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; MODULE      : menu-file.scm
;; DESCRIPTION : the file menus
;; COPYRIGHT   : (C) 1999  Joris van der Hoeven
;;
;; This software falls under the GNU general public license and comes WITHOUT
;; ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for details.
;; If you don't have this file, write to the Free Software Foundation, Inc.,
;; 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(texmacs-module (menus menu-file)
  (:use
    (texmacs texmacs tm-server) (texmacs texmacs tm-files)
    (texmacs texmacs tm-print)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Submenus of the File menu and for the icon bar
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(menu-bind new-file-menu
  ("New buffer" (new-buffer))
  ("Open new window" (open-window))
  ("Clone window" (clone-window)))

(menu-bind load-menu
  ("Load buffer" ... (choose-file "Load file" "" 'load-buffer))
  ("Revert buffer" ... (revert-buffer))
  ("Load in new window" ...
   (choose-file "Load file" "" 'load-in-new-window))
  ---
  ("Import#Verbatim" ...
   (choose-file "Load verbatim file" "" 'load-verbatim-buffer))
  ("Import#LaTeX" ...
   (choose-file "Load LaTeX file" "latex" 'load-latex-buffer))
  ("Import#HTML" ...
   (choose-file "Load html file" "html" 'load-html-buffer))
  ("Import#XML" ...
   (choose-file "Load xml file" "xml" 'load-xml-buffer))
  ("Import#Scheme" ...
   (choose-file "Load scheme file" "scheme" 'load-scheme-buffer)))

(menu-bind save-menu
  ("Save buffer" (save-buffer))
  ("Save buffer as" ...
   (choose-file "Save TeXmacs file" "TeXmacs" 'save-buffer))
  ---
  ("Export as#Verbatim" ...
   (choose-file "Save verbatim file" "verbatim" 'save-verbatim-buffer))
  ("Export as#LaTeX" ...
   (choose-file "Save LaTeX file" "latex" 'save-latex-buffer))
  ("Export as#HTML" ...
   (choose-file "Save html file" "html" 'save-html-buffer))
  ("Export as#XML" ...
   (choose-file "Save xml file" "xml" 'save-xml-buffer))
  ("Export as#Scheme" ...
   (choose-file "Save scheme file" "scheme" 'save-scheme-buffer))
  ("Export as#PostScript" ...
   (choose-file "Save postscript file" "postscript" 'print-to-file)))

(menu-bind print-menu
  ("Preview with ghostview" (preview-with-ghostview))
  ---
  ("Print all" (print))
  ("Print page selection" ...
   (interactive '("First page:" "Last page:") 'print-pages))
  ("Print all to file" ...
   (choose-file "Print all to file" "postscript" 'print-to-file))
  ("Print page selection to file" ...
   (interactive '("First page:" "Last page:")
		'choose-file-and-print-page-selection)))

(menu-bind close-menu
  ("Close buffer" (safely-kill-buffer))
  ("Close window" (safely-kill-window))
  ("Close TeXmacs" (safely-quit-TeXmacs)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; The File menu
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(menu-bind file-menu
  ("New" (new-buffer))
  ("Load" ... (choose-file "Load file" "" 'load-buffer))
  ("Save" (save-buffer))
  ("Save as" ...
   (choose-file "Save TeXmacs file" "TeXmacs" 'save-buffer))
  ("Revert" ... (revert-buffer))
  ---
  (-> "Page setup" (link page-setup-menu))
  (-> "Print" (link print-menu))
  (-> "Import"
      ("Verbatim" ...
       (choose-file "Load verbatim file" "" 'load-verbatim-buffer))
      ("LaTeX" ... (choose-file "Load LaTeX file" "latex"
				'load-latex-buffer))
      ("HTML" ... (choose-file "Load html file" "html" 'load-html-buffer))
      ("XML" ... (choose-file "Load xml file" "xml" 'load-xml-buffer))
      ("Scheme" ...
       (choose-file "Load scheme file" "scheme" 'load-scheme-buffer)))
  (-> "Export"
      ("Verbatim" ...
       (choose-file "Save verbatim file" "verbatim" 'save-verbatim-buffer))
      ("LaTeX" ...
       (choose-file "Save LaTeX file" "latex" 'save-latex-buffer))
      ("HTML" ...
       (choose-file "Save HTML file" "html" 'save-html-buffer))
      ("XML" ...
       (choose-file "Save XML file" "xml" 'save-xml-buffer))
      ("Scheme" ...
       (choose-file "Save scheme file" "scheme" 'save-scheme-buffer))
      ("PostScript" ...
       (choose-file "Save postscript file" "postscript" 'print-to-file)))
  ---
  ("Close buffer" (safely-kill-buffer))
  ("Close TeXmacs" (safely-quit-TeXmacs)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; The Go menu
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(menu-bind go-menu
  (link buffer-menu)
  (if (not (null? (menu-get 'bookmarks-menu)))
      ---
      (link bookmarks-menu)))
